﻿/*********************************************************************
 * Jexus Owin开放接口针对NancyFx的适配器
 * --------------------------------------------------------
 * 注：参考过Nancy有关自宿主及asp.net宿主有关源码。
 * 本适配器最初设计人为“宇内流云”，时间是2014年6月。
 * 本适配器开源，任何人都可以修改和自由使用。
 * 如有修改完善，请尽快将源码和修修说明反馈给宇内流云，以便统一更新。
 * *******************************************************************/


/************************************************************************
 * 使用方法：
 * 1、编译本工程，得到Jws.OwinAdapter.Nancy.dll。
 * 2、把这个dll和Nancy.dll、Nancy.Owin.dll放到网站的bin文件夹中。
 * 3、如果bin文件夹中有Nancy.Hosting.Aspnet.dll，请一定去掉。
 * 4、在需要启用Nanacy框架的网站的JWS配置文件中加入一行：
 *    OwinMain=Jws.OwinAdapter.Nancy.dll, Jws.OwinAdapter.Nancy.Adapter
 * 5、重启JWS，让配置生效。
 * ----------------------------------------------------------------------
 * 注：NancyFx完全可以通过Jws.OwinAdapter.MsOwin.dll按 MS OWIN 标准启动。
 *     之所以使用本适配器，是为了用户能进行更多的自定义配置。
 * **********************************************************************/


#region USINGs

using System.Collections.Generic;
using System.Threading.Tasks;
using Nancy.Bootstrapper;
using Nancy.Owin;

//for self bootstrapper
//using Nancy.Conventions;
//using Nancy;

#endregion



namespace Jws.OwinAdapter.Nancy
{

   
    /// <summary>
    /// 针对JWS开放接口与Nancy的适配器
    /// </summary>
    public class Adapter
    {
        /// <summary>
        /// NancyOwinHost对象，用于驱动Nancy处理请求
        /// </summary>
        private static NancyOwinHost _owinhost;

        /// <summary>
        /// 构造函数
        /// </summary>
        public Adapter()
        {
            //使用默认引导器
            var boot = NancyBootstrapperLocator.Bootstrapper;
            
            //使用自定义引导器
            //boot = new MyBootstrapper();
            
            //使用web.config中定义的引导器
            //var boot=WebConfigureBootstrapper.GetConfigurationBootstrapper() ?? NancyBootstrapperLocator.Bootstrapper;
            
            var options = new NancyOptions {Bootstrapper = boot, EnableClientCertificates = false};
            _owinhost = new NancyOwinHost(null, options);
        }



        /// <summary>
        /// 供JWS开放接口调用的关键方法
        /// <para>针对JWS开放接口必须实现这个方法</para>
        /// </summary>
        /// <param name="env">JWS调用本方法时提供的OWIN环境参数</param>
        /// <returns>返回一个服务</returns>
        public Task OwinMain(IDictionary<string, object> env)
        {

            /////////////////////////////////////////////////////////////////////
            // 强调：JWS OWIN适配器必须有这一个方法
            // 并且，这个方法的名称、参数类型、返回值类型不能改动。
            // 返回值只有两种可能性：
            // 一是null，表示这个请求不属于本框架处理（Nanacy不存在这个可能性）
            // 二是返回一个正在运行或已经运行完成的任务。
            //////////////////////////////////////////////////////////////////////////



            //驱动Nancy Owin App处理请求
            return _owinhost.Invoke(env);

        }

    }// End Class


    
    /*
    public class MyBootstrapper : DefaultNancyBootstrapper
    {
        protected override void ConfigureConventions(NancyConventions nancyConventions)
        {
            base.ConfigureConventions(nancyConventions);
            nancyConventions....、
            ............
        }
    }
    */



} //  End Namespace
