﻿/********************************************************
 * 声明：WebConfigureBootstrapper类参照于Nancy.Hosting.Aspnet有关代码。
 * **************************************************************************/
using System;
using Nancy.Bootstrapper;
using System.Configuration;


namespace Jws.OwinAdapter.Nancy
{
    /// <summary>
    /// 从Web.config中获取Bootstrapper配置信息的类
    /// </summary>
    class WebConfigureBootstrapper
    {

        /// <summary>
        /// 从web.config中NancyFx节获取自定义的NancyBootstrapper设置
        /// <para>如果没有这个节配置，将返回NULL</para>
        /// </summary>
        /// <returns></returns>
        public static INancyBootstrapper GetConfigurationBootstrapper()
        {
            var bootstrapperEntry = GetConfiguredBootstrapperEntry();
            if (bootstrapperEntry == null) return null;

            var assemblyQualifiedName = string.Concat(bootstrapperEntry.Name, ", ", bootstrapperEntry.Assembly);


            var bootstrapperType = Type.GetType(assemblyQualifiedName);
            if (bootstrapperType == null)
            {
                throw new BootstrapperException(string.Format("Could not locate bootstrapper of type '{0}'.", assemblyQualifiedName));
            }


            try
            {
                return Activator.CreateInstance(bootstrapperType) as INancyBootstrapper;
            }
            catch (Exception ex)
            {
                var errorMessage = string.Format("Could not initialize bootstrapper of type '{0}'.", bootstrapperType.FullName);
                throw new BootstrapperException(errorMessage, ex);
            }
        }


        private static BootstrapperEntry GetConfiguredBootstrapperEntry()
        {
            var configurationSection = ConfigurationManager.GetSection("nancyFx") as NancyFxSection;


            if (configurationSection == null) return null;

            var bootstrapperOverrideType = configurationSection.Bootstrapper.Type;
            var bootstrapperOverrideAssembly = configurationSection.Bootstrapper.Assembly;

            if (string.IsNullOrWhiteSpace(bootstrapperOverrideType))
            {
                throw new BootstrapperException("The 'type' attribute of the 'bootstrapper' element under the 'nancyFx' config section is empty. Please specify the full type name.");
            }


            if (string.IsNullOrWhiteSpace(bootstrapperOverrideAssembly))
            {
                throw new BootstrapperException("The 'assembly' attribute of the 'bootstrapper' element under the 'nancyFx' config section is empty. Please specify the full assembly name.");
            }


            return new BootstrapperEntry(bootstrapperOverrideAssembly, bootstrapperOverrideType);
        }


    }
}
